# -*- coding: utf-8 -*-
# -*- Python Version: 2.7 -*-

"""Function to parse input string from the user in numeric and 'unit' parts."""

try:
    from typing import Optional, Tuple, Union
except ImportError:
    pass  # IronPython 2.7

import re


def parse_input(_input_string):
    # type: (Union[str, int, float]) -> Tuple[str, Optional[str]]
    """Parse an input string into the 'value part' and the 'units part'.

    examples:
        '0.5 in' returns -> ('0.5', 'IN')
        '0.5" '  returns -> ('0.5', 'IN')
        '0.5'    returns -> ('0.5', None)
        0.5      returns -> ('0.5', None)
        '0.5Btu/hr-Ft2-F' returns -> ('0.5', 'BTU/HR-FT2-F')
        '45,567 ft2' returns -> ('45567', 'FT2')
    """

    # -- First, try and split the input string at the first alpha-character found
    # -- ie: the input string: "0.5 HR-FT2-F/BTU-IN" will get split at index=4
    # --                           ^
    # -- You can't just look for the numbers, since some units have numbers in them ('FT2', ...)
    # -- Exclude the '.' and '-' characters from the find so that float and
    # -- negative values don't cause an error.
    # --
    # -- https://regex101.com/

    input_string_cleaned = str(_input_string).strip().upper()
    input_string_cleaned = input_string_cleaned.replace("'", "FT").replace('"', "IN")
    input_string_cleaned = re.sub(r"(?<=\d),(?=\d)", "", input_string_cleaned)

    # -------------------------------------------------------------------------
    # -- Find the alpha and the numeric parts
    rx = re.compile(r"[^\d.-]", re.IGNORECASE)
    match = rx.search(input_string_cleaned)
    if not match:
        # -- No alpha part, just return the numeric part
        return (input_string_cleaned, None)
    found_span = match.span()

    # -------------------------------------------------------------------------
    # -- Cleanup
    alpha_part = input_string_cleaned[found_span[0] :].strip()
    numeric_part = input_string_cleaned[: found_span[0]].strip()

    return numeric_part, alpha_part



# -*- coding: utf-8 -*-
# -*- Python Version: 2.7 -*-

from ph_units.unit_types._base import Base_UnitType


class WattsPerMeterSquaredKelvin(Base_UnitType):
    """W/M2K (U-Value)"""

    __symbol__ = "W/M2K"
    __aliases__ = ["U-SI", "W/M²K", "W/M2-K", "W/M²-K"]
    __factors__ = {
        "W/M2K": "{}*1",
        "M2K/W": "(1/{})",
        "BTU/HR-FT2-F": "{}*0.176110159",
        "HR-FT2-F/BTU": "(1/{})*5.678264134",
    }


class BtuPerHourFootSquaredFahrenheit(Base_UnitType):
    """BTU/HR-FT2-F (U-Value)"""

    __symbol__ = "BTU/HR-FT2-F"
    __aliases__ = ["U-IP", "BTU/HR-SF-F", "BTU/H-SF-F", "BTU/HRFT²°F"]
    __factors__ = {
        "W/M2K": "{}*5.678264134",
        "M2K/W": "1/({}*5.678264134)",
        "BTU/HR-FT2-F": "{}*1",
        "HR-FT2-F/BTU": "(1/{})",
    }


class MeterSquaredKelvinPerWatt(Base_UnitType):
    """M2K/W (R-Value)"""

    __symbol__ = "M2K/W"
    __aliases__ = ["R-SI", "M²K/W"]
    __factors__ = {
        "M2K/W": "{}*1",
        "W/M2K": "(1/{})",
        "HR-FT2-F/BTU": "{}*5.678264134",
        "BTU/HR-FT2-F": "1/({}*5.678264134)",
    }


class HourFootSquaredFahrenheitPerBtu(Base_UnitType):
    """HR-FT2-F/BTU (R-Value)"""

    __symbol__ = "HR-FT2-F/BTU"
    __aliases__ = ["R-IP", "H-SF-F/BTU", "HR-SF-F/BTU"]
    __factors__ = {
        "M2K/W": "1/((1/{})*5.678264134)",
        "W/M2K": "(1/{})*5.678264134",
        "HR-FT2-F/BTU": "{}*1",
        "BTU/HR-FT2-F": "(1/{})",
    }


class HourFootSquaredFahrenheitPerBtuInch(Base_UnitType):
    """HR-FT2-F/BTU-IN (R-per-inch)"""

    __symbol__ = "HR-FT2-F/BTU-IN"
    __aliases__ = ["R/IN", "R-IN", "H-SF-F/BTU-IN", "R/INCH"]
    __factors__ = {
        "W/MK": "(1/({}*12))*1.730734908",
        "BTU/HR-FT-F": "1/({}*12)",
    }


class WattsPerMeterKelvin(Base_UnitType):
    """W/MK (Psi-Value)"""

    __symbol__ = "W/MK"
    __aliases__ = []
    __factors__ = {
        "W/MK": "{}*1",
        "HR-FT2-F/BTU-IN": "{}**-1*0.144227909",
        "BTU/HR-FT-F": "{}*0.577789236",
    }


class BtuPerHourFootFahrenheit(Base_UnitType):
    """BTU/HR-FT-F (Psi-Value)"""

    __symbol__ = "BTU/HR-FT-F"
    __aliases__ = ["BTU/HRFT°F"]
    __factors__ = {
        "W/MK": "{}*1.730734908",
        "HR-FT2-F/BTU-IN": "1/({}*12)",
        "BTU/HR-FT-F": "{}*1",
    }


class WattsPerKelvin(Base_UnitType):
    """W/K (Chi-Value)"""

    __symbol__ = "W/K"
    __aliases__ = []
    __factors__ = {"W/K": "{}*1", "BTU/HR-F": "{}*1.895633976"}


class BtuPerHourFahrenheit(Base_UnitType):
    """BTU/HR-F (Chi-Value)"""

    __symbol__ = "BTU/HR-F"
    __aliases__ = ["BTU/HRF", "BTU/HR-F", "BTU/HR-F", "BTU/HR°F", "BTU/HR-°F"]
    __factors__ = {"W/K": "{}*0.527528", "BTU/HR-F": "{}*1"}
